import hashlib
import os
import typing
from pathlib import Path


def rm_duplicate_hashes(
    files: typing.List[Path],
    date_modified: bool = False,
    reverse: bool = False,
) -> None:
    """Iterate through a list of paths and remove those with duplicate
    hashes.

    Args:
        files (typing.Path[List]): A list of file paths.
        date_modified (bool, optional): Flag to select the remaining
        copy by most recently modified. Defaults to False (the
        filename).
        reverse (bool, optional): Flag to reverse the selection process.
        Defaults to False.
    """
    # If the date modified flag is True, then sort by the date modified.
    # NOTE: Perhaps counterintuitively, reverse st_mtime is actually
    # what we want to represent the date last modified.
    if date_modified:
        if reverse:
            files.sort(key=lambda x: x.stat().st_mtime, reverse=False)
        else:
            files.sort(key=lambda x: x.stat().st_mtime, reverse=True)
    # Else, sort by filename.
    else:
        if reverse:
            files.sort(key=lambda x: x.name, reverse=True)
        else:
            files.sort(key=lambda x: x.name, reverse=False)

    hash_set = set()  # Set of all hashes.

    unique_files = []
    for file in files:
        if file not in unique_files:
            unique_files.append(file)

    # Iterate through every unique file in files.
    for file in unique_files:
        try:
            with open(file, "rb") as f:
                file_bytes = f.read()
                hash = hashlib.sha256(file_bytes).hexdigest()

                if hash in hash_set:
                    os.remove(file)
                else:
                    hash_set.add(hash)

        except Exception as e:
            raise e
